# -*- coding: utf-8 -*-

from collective.behavior.internalnumber import TYPE_CONFIG
from collective.behavior.internalnumber.browser.settings import get_pt_settings
from collective.behavior.internalnumber.subscribers import object_added
from collective.behavior.internalnumber.testing import COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING  # noqa
from plone import api

import unittest


class TestSubscribers(unittest.TestCase):
    """Test settings."""

    layer = COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.tt1 = self.portal['tt1']
        api.portal.set_registry_record(TYPE_CONFIG, [{u'portal_type': u'glo_bal', u'uniqueness': False,
                                                      u'default_number': 1, u'default_expression': u'number'},
                                                     {u'portal_type': u'testtype', u'uniqueness': True,
                                                      u'default_number': 1, u'default_expression': u'number'}])

    def test_object_added(self):
        self.assertDictEqual(get_pt_settings(u'testtype'), {u'u': True, u'nb': 1, 'expr': u'number'})
        self.assertDictEqual(get_pt_settings(u'glo_bal'), {u'u': False, u'nb': 1, 'expr': u'number'})
        # test with testtype
        object_added(self.tt1, None)
        self.assertDictEqual(get_pt_settings(u'testtype'), {u'u': True, u'nb': 2, 'expr': u'number'})
        self.assertDictEqual(get_pt_settings(u'glo_bal'), {u'u': False, u'nb': 1, 'expr': u'number'})

        class Dummy(object):
            def __init__(self, val=None, pt='unknown'):
                self.internal_number = val
                self.portal_type = pt
        # test with other type
        object_added(Dummy(val='xxx'), None)
        self.assertDictEqual(get_pt_settings(u'testtype'), {u'u': True, u'nb': 2, 'expr': u'number'})
        self.assertDictEqual(get_pt_settings(u'glo_bal'), {u'u': False, u'nb': 2, 'expr': u'number'})
        # test with empty value
        object_added(Dummy(), None)
        self.assertDictEqual(get_pt_settings(u'testtype'), {u'u': True, u'nb': 2, 'expr': u'number'})
        self.assertDictEqual(get_pt_settings(u'glo_bal'), {u'u': False, u'nb': 2, 'expr': u'number'})
        # test with empty config
        api.portal.set_registry_record(TYPE_CONFIG, [])
        object_added(Dummy(self.tt1), None)
        self.assertDictEqual(get_pt_settings(u'testtype'), {})
        self.assertDictEqual(get_pt_settings(u'glo_bal'), {})



# -*- coding: utf-8 -*-
"""Init and utils."""
from plone import api
from zope.i18nmessageid import MessageFactory

import logging


logger = logging.getLogger('collective.behavior.internalnumber')
_ = MessageFactory('collective.behavior.internalnumber')

REGISTRY_KEY = 'collective.behavior.internalnumber.browser.settings.IInternalNumberConfig'
TYPE_CONFIG = '%s.portal_type_config' % REGISTRY_KEY

PLONE_VERSION = api.env.plone_version()[:3]



# -*- coding: utf-8 -*-

from collective.behavior.internalnumber import logger
from collective.behavior.internalnumber import TYPE_CONFIG
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import INonInstallable
from ZODB.POSException import ConnectionStateError
from zope.component import getUtility
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller"""
        return [
            'collective.behavior.internalnumber:uninstall',
        ]


def post_install(context):
    """Post install script"""
    registry = getUtility(IRegistry)
    # Initialize the registry content if nothing is stored
    if registry[TYPE_CONFIG] is None:
        # may fail in tests because a datagridfield is stored, just pass in this case
        try:
            registry[TYPE_CONFIG] = []
        except ConnectionStateError:
            logger.warn('!!!Failed to set registry config to []!!!')
            registry.records[TYPE_CONFIG].field.value_type = None


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.



# -*- coding: utf-8 -*-
"""Subscribers."""

from collective.behavior.internalnumber.browser.settings import increment_nb_for


def object_added(obj, event):
    """Increments the registry default_number for the type or globally (if one of both configured)."""
    increment_nb_for(obj)



# -*- coding: utf-8 -*-
from plone import api
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.testing import z2

import collective.behavior.internalnumber


class CollectiveBehaviorInternalnumberLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=collective.behavior.internalnumber, name='testing.zcml')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.behavior.internalnumber:testing')
        setRoles(portal, TEST_USER_ID, ['Manager'])
        content = api.content.create(container=portal, id='tt1', type='testtype', title='My content',
                                     internal_number='AA123')
        content.reindexObject()


COLLECTIVE_BEHAVIOR_INTERNALNUMBER_FIXTURE = CollectiveBehaviorInternalnumberLayer()


COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_BEHAVIOR_INTERNALNUMBER_FIXTURE,),
    name='CollectiveBehaviorInternalnumberLayer:IntegrationTesting'
)


COLLECTIVE_BEHAVIOR_INTERNALNUMBER_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_BEHAVIOR_INTERNALNUMBER_FIXTURE,),
    name='CollectiveBehaviorInternalnumberLayer:FunctionalTesting'
)


COLLECTIVE_BEHAVIOR_INTERNALNUMBER_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVE_BEHAVIOR_INTERNALNUMBER_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name='CollectiveBehaviorInternalnumberLayer:AcceptanceTesting'
)



# -*- coding: utf-8 -*-

from collective.behavior.internalnumber import PLONE_VERSION
from collective.behavior.internalnumber.behavior import IInternalNumberBehavior
from plone.indexer import indexer
from Products.CMFCore.interfaces import IContentish
from Products.CMFPlone.utils import base_hasattr
from Products.CMFPlone.utils import safe_unicode
from zope.component import adapts
from zope.interface import implementer


try:
    from Products.PluginIndexes.common.UnIndex import _marker as common_marker  # noqa
except ImportError:
    from Products.PluginIndexes.unindex import _marker as common_marker  # noqa

if PLONE_VERSION < '6.0':
    from collective.dexteritytextindexer.interfaces import IDynamicTextIndexExtender
else:
    from plone.app.dexterity.textindexer.interfaces import IDynamicTextIndexExtender


@indexer(IContentish)
def internal_number_index(obj):
    """ Index method escaping acquisition and ready for ZCatalog 3 """
    if base_hasattr(obj, 'internal_number') and obj.internal_number:
        return obj.internal_number
    return common_marker


@implementer(IDynamicTextIndexExtender)
class InternalNumberSearchableExtender(object):
    """
        Extends SearchableText of IInternalNumberBehavior objects.
    """
    adapts(IInternalNumberBehavior)

    def __init__(self, context):
        self.context = context

    def __call__(self):
        return safe_unicode(self.context.internal_number)



# -*- coding: utf-8 -*-

from collective.behavior.internalnumber import _
from collective.behavior.internalnumber.browser.settings import get_pt_settings
from collective.behavior.talcondition.utils import _evaluateExpression
from plone import api
from plone.autoform.interfaces import IFormFieldProvider
from plone.supermodel import model
from Products.CMFPlone.utils import safe_unicode
from z3c.form import validator
from z3c.form import widget
from zope import component
from zope import schema
from zope.interface import alsoProvides
from zope.interface import Invalid


class IInternalNumberBehavior(model.Schema):

    internal_number = schema.TextLine(
        title=_(u"Internal number"),
        required=False,)


alsoProvides(IInternalNumberBehavior, IFormFieldProvider)


def validateIndexValueUniqueness(context, portal_type, index_name, value):
    """
        check at 'portal_type' 'context' creation if 'index' 'value' is uniqueness
    """
    # if the value is empty, we don't check anything
    if not value:
        return
    catalog = api.portal.get_tool('portal_catalog')
    brains = catalog.searchResults(**{index_name: value})
    if context.portal_type != portal_type:
        # we create the object, the context is the container
        if brains:
            raise Invalid(_(u"This value '${value}' is already used in ${obj}",
                            mapping={'obj': ', '.join(['<a href="%s" target="_blank">%s</a>' %
                                                       (safe_unicode(b.getURL()), safe_unicode(b.Title))
                                                       for b in brains]),
                                     'value': safe_unicode(value)}))
    else:
        # we edit the object, the context is itself
        # if multiple brains (normally not possible), we are sure there are other objects with same index value
        if len(brains) > 1 or (len(brains) == 1 and brains[0].UID != context.UID()):
            raise Invalid(_(u"This value '${value}' is already used in ${obj}",
                            mapping={'obj': ', '.join(['<a href="%s" target="_blank">%s</a>' %
                                                       (safe_unicode(b.getURL()), safe_unicode(b.Title))
                                                       for b in brains]),
                                     'value': safe_unicode(value)}))


class InternalNumberValidator(validator.SimpleFieldValidator):
    def validate(self, value):
        # we call the already defined validators
        # super(InternalNumberValidator, self).validate(value)
        if not get_pt_settings(self.view.portal_type).get('u', True):
            return
        validateIndexValueUniqueness(self.context, self.view.portal_type, 'internal_number', value)


validator.WidgetValidatorDiscriminators(InternalNumberValidator, field=IInternalNumberBehavior['internal_number'])
component.provideAdapter(InternalNumberValidator)


# To avoid grokking the package required by plone.directives.form.default_value decorator

def internal_number_default(data):
    """ Default value of internal_number """
    if data.view is None or not hasattr(data.view, 'portal_type'):
        return u''
    settings = get_pt_settings(data.view.portal_type)
    ret = _evaluateExpression(data.context, settings.get('expr', ''), extra_expr_ctx={'number': settings.get('nb', 0)},
                              empty_expr_is_true='')
    return ret


DefaultinternalNumber = widget.ComputedWidgetAttribute(internal_number_default,
                                                       field=IInternalNumberBehavior['internal_number'])

component.provideAdapter(DefaultinternalNumber, name='default')



# -*- coding: utf-8 -*-

from collective.behavior.internalnumber import _
from collective.behavior.internalnumber import TYPE_CONFIG
from collective.behavior.talcondition.utils import _evaluateExpression
from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.registry import DictRow
from plone import api
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.autoform.directives import widget
from plone.dexterity.interfaces import IDexterityFTI
from plone.z3cform import layout
from Products.CMFPlone.utils import base_hasattr
from z3c.form import form
from zope import schema
from zope.component import getAllUtilitiesRegisteredFor
from zope.component import getUtility
from zope.i18n.interfaces import ITranslationDomain
from zope.interface import implementer
from zope.interface import Interface
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class IPortalTypeConfigSchema(Interface):
    portal_type = schema.Choice(
        title=_("Portal type"),
        vocabulary=u'collective.internalnumber.portaltypevocabulary',
        required=True)
    uniqueness = schema.Bool(
        title=_("Uniqueness"),
        required=False)
    default_number = schema.Int(
        title=_(u'Number of next content item'),
        description=_(u"This value can be used as 'number' variable in tal expression"),
        default=1)
    default_expression = schema.TextLine(
        title=_("Default value tal expression"),
        description=_("Elements 'number', 'member', 'context' and 'portal' are available."),
        default=u"number",
        required=False)


class IInternalNumberConfig(Interface):
    """
    Configuration of internalnumber
    """

    portal_type_config = schema.List(
        title=_(u'By type configuration'),
        value_type=DictRow(title=_("Portal type conf"),
                           schema=IPortalTypeConfigSchema))

    widget('portal_type_config', DataGridFieldFactory)


class SettingsEditForm(RegistryEditForm):
    """
    Define form logic
    """
    form.extends(RegistryEditForm)
    schema = IInternalNumberConfig
    label = _("Internal number behavior configuration")


SettingsView = layout.wrap_form(SettingsEditForm, ControlPanelFormWrapper)


def get_settings():
    ptc = api.portal.get_registry_record(TYPE_CONFIG)
    settings = {}
    if ptc is None:
        return settings
    for row in ptc:
        expr = row['default_expression'] and row['default_expression'] or u''
        settings[row['portal_type']] = {'u': row['uniqueness'], 'nb': row['default_number'], 'expr': expr}
    return settings


def get_pt_settings(pt):
    settings = get_settings()
    if pt in settings:
        return settings[pt]
    elif 'glo_bal' in settings:
        return settings['glo_bal']
    return {}


def set_settings(settings):
    config = []
    for pt in sorted(settings.keys()):
        config.append({'portal_type': pt, 'uniqueness': settings[pt]['u'],
                       'default_number': settings[pt]['nb'],
                       'default_expression': settings[pt]['expr']})
    api.portal.set_registry_record(TYPE_CONFIG, config)


def _internal_number_is_used(obj):
    """ """
    return base_hasattr(obj, 'internal_number') and obj.internal_number


def increment_nb_for(obj, bypass_attr_check=False):
    # internal_number is unknown or empty => no need to increment
    if not bypass_attr_check and not _internal_number_is_used(obj):
        return

    settings = get_settings()
    pt = obj.portal_type
    updated = False
    nb = None
    if pt in settings:
        updated = True
        settings[pt]['nb'] += 1
        nb = settings[pt]['nb']
    elif 'glo_bal' in settings:
        updated = True
        settings['glo_bal']['nb'] += 1
        nb = settings['glo_bal']['nb']
    if updated:
        set_settings(settings)
    return nb


def decrement_nb_for(obj):
    # internal_number is unknown or empty => no need to decrement
    if not _internal_number_is_used(obj):
        return

    settings = get_settings()
    pt = obj.portal_type
    updated = False
    nb = None
    if pt in settings:
        updated = True
        settings[pt]['nb'] -= 1
        nb = settings[pt]['nb']
    elif 'glo_bal' in settings:
        updated = True
        settings['glo_bal']['nb'] -= 1
        nb = settings['glo_bal']['nb']
    if updated:
        set_settings(settings)
    return nb


def decrement_if_last_nb(obj):
    # internal_number is unknown or empty => no need to decrement
    if not _internal_number_is_used(obj):
        return

    internal_number = getattr(obj, "internal_number")
    settings = get_settings()
    pt = obj.portal_type
    updated = False
    nb = None

    def _compute_expr(obj, pt_settings):
        return _evaluateExpression(
            obj,
            pt_settings['expr'],
            extra_expr_ctx={'number': pt_settings['nb'] - 1},
            empty_expr_is_true='')

    if pt in settings and internal_number == _compute_expr(obj, settings[pt]):
        updated = True
        settings[pt]['nb'] -= 1
        nb = settings[pt]['nb']
    elif 'glo_bal' in settings and \
            internal_number == _compute_expr(obj, settings['glo_bal']):
        updated = True
        settings['glo_bal']['nb'] -= 1
        nb = settings['glo_bal']['nb']
    if updated:
        set_settings(settings)
    return nb


@implementer(IVocabularyFactory)
class DxPortalTypesVocabulary(object):
    """ Active mail types vocabulary """

    def __call__(self, context):
        terms = [SimpleTerm('glo_bal', 'glo_bal', _(u'Global configuration'))]
        ftis = getAllUtilitiesRegisteredFor(IDexterityFTI)
        portal = api.portal.get()
        for fti in ftis:
            translation_domain = getUtility(ITranslationDomain, fti.i18n_domain)
            terms.append(SimpleTerm(fti.id, fti.id, translation_domain.translate(
                fti.title, context=portal.REQUEST)))
        return SimpleVocabulary(terms)



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from collective.behavior.internalnumber import PLONE_VERSION
from collective.behavior.internalnumber import TYPE_CONFIG
from collective.behavior.internalnumber.testing import COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING
from plone import api
from zope.schema._bootstrapinterfaces import WrongType

import unittest


class CommonSetup():  # noqa

    def set_installer(self):
        if PLONE_VERSION >= '5.1':
            from Products.CMFPlone.utils import get_installer  # noqa
            self.installer = get_installer(self.portal, self.layer["request"])
            self.ipi = self.installer.is_product_installed
        else:
            self.installer = api.portal.get_tool('portal_quickinstaller')  # noqa
            self.ipi = self.installer.isProductInstalled


class TestSetup(unittest.TestCase, CommonSetup):
    """Test that collective.behavior.internalnumber is properly installed."""

    layer = COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.set_installer()

    def test_product_installed(self):
        """Test if collective.behavior.internalnumber is installed."""
        self.assertTrue(self.ipi('collective.behavior.internalnumber'))

    def test_browserlayer(self):
        """Test that ICollectiveBehaviorInternalnumberLayer is registered."""
        from collective.behavior.internalnumber.interfaces import ICollectiveBehaviorInternalnumberLayer
        from plone.browserlayer import utils
        self.assertIn(ICollectiveBehaviorInternalnumberLayer, utils.registered_layers())

    def test_registry(self):
        self.assertRaises(api.exc.InvalidParameterError, api.portal.set_registry_record, 'Unexistent key', True)
        self.assertRaises(WrongType, api.portal.set_registry_record, TYPE_CONFIG, 'string')
        api.portal.set_registry_record(TYPE_CONFIG, [])

    def test_catalog(self):
        catalog = api.portal.get_tool('portal_catalog')
        # we have an index
        self.assertTrue('internal_number' in catalog.Indexes)
        # we no more have a metadata
        self.assertFalse('internal_number' in catalog.schema())


class TestUninstall(unittest.TestCase, CommonSetup):

    layer = COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.set_installer()
        if PLONE_VERSION >= '5.1':
            self.installer.uninstall_product('collective.behavior.internalnumber')
        else:
            self.installer.uninstallProducts(['collective.behavior.internalnumber'])

    def test_product_uninstalled(self):
        """Test if collective.behavior.internalnumber is cleanly uninstalled."""
        self.assertFalse(self.ipi('collective.behavior.internalnumber'))

    def test_browserlayer_removed(self):
        """Test that ICollectiveBehaviorInternalnumberLayer is removed."""
        from collective.behavior.internalnumber.interfaces import ICollectiveBehaviorInternalnumberLayer
        from plone.browserlayer import utils
        self.assertNotIn(ICollectiveBehaviorInternalnumberLayer, utils.registered_layers())

    def test_catalog_cleaned(self):
        catalog = api.portal.get_tool('portal_catalog')
        # no more catalog index
        self.assertFalse('internal_number' in catalog.Indexes)
        # no more catalog metadata
        self.assertFalse('internal_number' in catalog.schema())



# -*- coding: utf-8 -*-

from plone import api


def upgrade_to_1001(context):
    """Upgrade to 1001, remove the "internal_number" catalog metadata."""

    catalog = api.portal.get_tool('portal_catalog')
    catalog.delColumn('internal_number')



# -*- coding: utf-8 -*-

from collective.behavior.internalnumber import PLONE_VERSION
from collective.behavior.internalnumber import TYPE_CONFIG
from collective.behavior.internalnumber.behavior import internal_number_default
from collective.behavior.internalnumber.behavior import InternalNumberValidator
from collective.behavior.internalnumber.behavior import validateIndexValueUniqueness
from collective.behavior.internalnumber.testing import COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING  # noqa
from plone import api
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import getUtility
from zope.interface import Invalid

import unittest


class TestBehavior(unittest.TestCase):
    """Test behavior."""

    layer = COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.tt1 = self.portal['tt1']
        self.tt2 = api.content.create(container=self.portal, id='tt2', type='testtype', title='Another title')
        self.tt2.reindexObject()
        api.portal.set_registry_record(TYPE_CONFIG, [{u'portal_type': u'testtype', u'uniqueness': True,
                                                      u'default_number': 1, u'default_expression': u'number'}])

    def test_content(self):
        self.assertEqual(self.tt1.internal_number, 'AA123')
        self.assertEqual(self.tt2.internal_number, None)

    def test_index(self):
        pc = self.portal.portal_catalog
        brains = pc(portal_type='testtype')
        self.assertEqual(len(brains), 2)
        brains = pc(portal_type='testtype', internal_number='AA123')
        self.assertEqual(len(brains), 1)

    def test_searchabletext_index(self):
        fti = getUtility(IDexterityFTI, name='testtype')
        behaviors = list(fti.behaviors)
        if PLONE_VERSION < '6.0':
            behaviors.append('collective.dexteritytextindexer.behavior.IDexterityTextIndexer')
        else:
            behaviors.append('plone.textindexer')
        fti._updateProperty('behaviors', tuple(behaviors))
        self.tt1.reindexObject()
        pc = self.portal.portal_catalog
        brains = pc(portal_type='testtype', SearchableText='AA123')
        self.assertEqual(len(brains), 1)
        self.assertEqual(brains[0].getObject(), self.tt1)
        brains = pc(portal_type='testtype', SearchableText='content')
        self.assertEqual(len(brains), 1)
        self.assertEqual(brains[0].getObject(), self.tt1)

    def test_validator(self):
        self.assertRaises(Invalid, validateIndexValueUniqueness, self.tt2, 'testtype', 'internal_number', 'AA123')
        self.assertIsNone(validateIndexValueUniqueness(self.tt2, 'testtype', 'internal_number', 'AA1234'))
        # context, request, view, field, widget
        # make validator with tt2 as context and tt2 as portal_type
        validator = InternalNumberValidator(self.tt2, None, self.tt2, None, None)
        # validate following configuration: flag is True
        self.assertRaises(Invalid, validator.validate, 'AA123')
        self.assertIsNone(validator.validate('AA1234'))
        # validate following configuration: flag is not defined
        api.portal.set_registry_record(TYPE_CONFIG, [])
        self.assertRaises(Invalid, validator.validate, 'AA123')
        self.assertIsNone(validator.validate('AA1234'))
        # validate following configuration: flag is globally defined
        api.portal.set_registry_record(TYPE_CONFIG, [{u'portal_type': u'glo_bal', u'uniqueness': False,
                                                      u'default_number': 1, u'default_expression': u'number'}])
        self.assertIsNone(validator.validate('AA123'))
        # validate following configuration: flag is False
        api.portal.set_registry_record(TYPE_CONFIG, [{u'portal_type': u'testtype', u'uniqueness': False,
                                                      u'default_number': 1, u'default_expression': u'number'}])
        self.assertIsNone(validator.validate('AA123'))
        # make validator with portal as context and tt2 as portal_type
        validator = InternalNumberValidator(self.portal, None, self.tt2, None, None)
        api.portal.set_registry_record(TYPE_CONFIG, [])
        self.assertRaises(Invalid, validator.validate, 'AA123')
        self.assertIsNone(validator.validate('AA1234'))

    def test_internal_number_default(self):
        class Dummy(object):
            def __init__(self, obj):
                self.context = obj
                self.portal_type = obj.portal_type
                self.view = None
        dummy1 = Dummy(self.tt1)
        dummy1.view = dummy1
        self.assertEqual(internal_number_default(dummy1), 1)
        api.portal.set_registry_record(TYPE_CONFIG, [{u'portal_type': u'testtype', u'uniqueness': False,
                                                      u'default_number': 1,
                                                      u'default_expression': u"python: 'Num: %d' % (number+1)"}])
        self.assertEqual(internal_number_default(dummy1), 'Num: 2')



# -*- coding: utf-8 -*-
from collective.behavior.internalnumber import PLONE_VERSION
from collective.behavior.internalnumber import TYPE_CONFIG
from collective.behavior.internalnumber.browser.settings import decrement_if_last_nb
from collective.behavior.internalnumber.browser.settings import decrement_nb_for
from collective.behavior.internalnumber.browser.settings import DxPortalTypesVocabulary
from collective.behavior.internalnumber.browser.settings import get_pt_settings
from collective.behavior.internalnumber.browser.settings import get_settings
from collective.behavior.internalnumber.browser.settings import increment_nb_for
from collective.behavior.internalnumber.browser.settings import set_settings
from collective.behavior.internalnumber.testing import COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING  # noqa
from operator import itemgetter
from plone import api

import unittest


class TestSettings(unittest.TestCase):
    """Test settings."""

    layer = COLLECTIVE_BEHAVIOR_INTERNALNUMBER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.tt1 = self.portal['tt1']
        api.portal.set_registry_record(TYPE_CONFIG, [{u'portal_type': u'glo_bal', u'uniqueness': False,
                                                      u'default_number': 1, u'default_expression': u'number'},
                                                     {u'portal_type': u'testtype', u'uniqueness': True,
                                                      u'default_number': 1, u'default_expression': u'number'}])

    def test_get_settings(self):
        self.assertDictEqual(get_settings(), {u'glo_bal': {u'u': False, u'nb': 1, 'expr': u'number'},
                                              u'testtype': {u'u': True, u'nb': 1, 'expr': u'number'}})

    def test_get_pt_settings(self):
        self.assertDictEqual(get_pt_settings('testtype'), {u'u': True, u'nb': 1, 'expr': u'number'})
        self.assertDictEqual(get_pt_settings('unknown'), {u'u': False, u'nb': 1, 'expr': u'number'})
        api.portal.set_registry_record(TYPE_CONFIG, [{u'portal_type': u'glo_bal', u'uniqueness': False,
                                                      u'default_number': 1, u'default_expression': u'number'}])
        self.assertDictEqual(get_pt_settings('testtype'), {u'u': False, u'nb': 1, 'expr': u'number'})

    def test_DxPortalTypesVocabulary(self):
        voc_inst = DxPortalTypesVocabulary()
        voc_list = sorted([(t.value, t.title) for t in voc_inst(self.portal)], key=itemgetter(1))
        if PLONE_VERSION == '4.3':
            res = [('glo_bal', u'Global configuration'), ('testtype', u'Test type')]
        elif PLONE_VERSION == '5.2':
            res = [('Collection', 'Collection'), ('Event', 'Event'), ('File', 'File'), ('Folder', 'Folder'),
                   ('glo_bal', 'Global configuration'), ('Image', 'Image'), ('Link', 'Link'),
                   ('News Item', 'News Item'), ('Document', 'Page'), ('testtype', 'Test type')]
        elif PLONE_VERSION > '5.2':
            res = [('Collection', 'Collection'), ('Event', 'Event'), ('File', 'File'), ('Folder', 'Folder'),
                   ('glo_bal', 'Global configuration'), ('Image', 'Image'), ('Link', 'Link'),
                   ('News Item', 'News Item'), ('Document', 'Page'), ('Plone Site', 'Plone Site'),
                   ('testtype', 'Test type')]
        self.assertEqual(voc_list, res)

    def test_increment_nb_for(self):
        self.assertEqual(get_pt_settings(self.tt1.portal_type), {'expr': u'number', 'nb': 1, 'u': True})
        self.assertEqual(increment_nb_for(self.tt1), 2)
        self.assertEqual(get_pt_settings(self.tt1.portal_type), {'expr': u'number', 'nb': 2, 'u': True})
        # glo_bal
        settings = get_settings()
        settings.pop(self.tt1.portal_type)
        set_settings(settings)
        self.assertEqual(increment_nb_for(self.tt1), 2)

    def test_decrement_nb_for(self):
        self.assertEqual(get_pt_settings(self.tt1.portal_type), {'expr': u'number', 'nb': 1, 'u': True})
        self.assertEqual(increment_nb_for(self.tt1), 2)
        self.assertEqual(get_pt_settings(self.tt1.portal_type), {'expr': u'number', 'nb': 2, 'u': True})
        self.assertEqual(decrement_nb_for(self.tt1), 1)
        self.assertEqual(get_pt_settings(self.tt1.portal_type), {'expr': u'number', 'nb': 1, 'u': True})
        # glo_bal
        settings = get_settings()
        settings.pop(self.tt1.portal_type)
        settings['glo_bal']['nb'] = 5
        set_settings(settings)
        self.assertEqual(decrement_nb_for(self.tt1), 4)
        # internal_number attr not used
        self.assertIsNone(decrement_nb_for(self.portal))

    def test_decrement_if_last_nb(self):
        self.tt1.internal_number = 1
        self.assertEqual(get_pt_settings(self.tt1.portal_type), {'expr': u'number', 'nb': 1, 'u': True})
        self.assertEqual(increment_nb_for(self.tt1), 2)
        self.assertEqual(get_pt_settings(self.tt1.portal_type), {'expr': u'number', 'nb': 2, 'u': True})
        # change tt1 internal_number is not the last, not decremented
        self.tt1.internal_number = 3
        self.assertIsNone(decrement_if_last_nb(self.tt1))
        # set internal_number as last nb
        self.tt1.internal_number = 1
        self.assertEqual(decrement_if_last_nb(self.tt1), 1)
        # internal_number attr not used
        self.assertIsNone(decrement_if_last_nb(self.portal))

    def test_decrement_if_last_nb_complex_internal_number(self):
        self.assertEqual(self.tt1.internal_number, 'AA123')
        settings = get_settings()
        settings[self.tt1.portal_type]['nb'] = 124
        settings[self.tt1.portal_type]['expr'] = u"string:AA${number}"
        set_settings(settings)
        self.assertEqual(decrement_if_last_nb(self.tt1), 123)
        # glo_bal
        settings.pop(self.tt1.portal_type)
        settings['glo_bal']['nb'] = 111
        settings['glo_bal']['expr'] = u"string:AA${number}"
        set_settings(settings)
        self.tt1.internal_number = 'AA110'
        self.assertEqual(decrement_if_last_nb(self.tt1), 110)
